<?php

namespace WebsiteBundle\Controller\Site;

use Bazookas\CommonBundle\Exception\NotImplementedException;
use Bazookas\CommonBundle\Utility\Utility;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Security\Core\Exception\BadCredentialsException;
use WebsiteBundle\Entity\Participant;
use WebsiteBundle\Form\Site\ForgotPasswordForm;
use WebsiteBundle\Form\Site\ResetPasswordForm;
use WebsiteBundle\Security\Roles;

class SecurityController extends Controller
{

  public function loginAction() {
    if ($this->isGranted(Roles::ROLE_PARTICIPANT)) {
      return $this->redirectToRoute('user_home');
    }

    $authenticationUtils = $this->get('security.authentication_utils');

    // get the login error if there is one
    $error = $authenticationUtils->getLastAuthenticationError();

    if (!empty($error)) {
      if ($error instanceof BadCredentialsException) {
        $translator = $this->get('translator');
        $message = $translator->trans('site.forms.login.errors.badCredentials');
      } else {
        $translator = $this->get('translator');
        $message = $translator->trans('site.forms.login.errors.systemError');
      }
      $this->addFlash('error', $message);
    }

    // last username entered by the user
    $lastUsername = $authenticationUtils->getLastUsername();

    return $this->render('@Website/Security/login.html.twig', [
      'last_username' => $lastUsername,
      'error' => $error,
      'headerTitle' => 'PREVEND'
    ]);
  }

  public function forgotPasswordAction(Request $request) {

    $form = $this->createForm(ForgotPasswordForm::class);

    $form->handleRequest($request);

    if ($form->isValid() && $form->isSubmitted()) {
      $email = $form->getData()['email'];

      $entityManager = $this->get('doctrine.orm.entity_manager');
      $entityRepo = $entityManager->getRepository(Participant::class);
      $participant = $entityRepo->findOneBy(['email' => $email]);

      $translator = $this->get('translator');
      if (!empty($participant)) {
        //Generate and persist the forgot hash
        $hash = urlencode(Utility::generateRandomString(64));
        $participant->setForgotPasswordHash($hash);
        $entityRepo->updateEntities([$participant]);

        //Generate the forgot password url
        $forgotLink = $this->generateUrl('user_reset_password', [
          'participantId' => $participant->getId(),
          'hash' => $participant->getForgotPasswordHash()
        ], UrlGeneratorInterface::ABSOLUTE_URL);

        //Now send them a validation email
        $emailService = $this->get('WebsiteBundle.services.EmailService');
        $emailService->sendForgotPasswordEmail($participant, $forgotLink);

        //Add a flash message
        $message = $translator->trans('site.forms.forgotPassword.success.emailSent', [
          '%email%' => $participant->getEmail()
        ]);
        $this->addFlash('success', $message);
      } else {
        $message = $translator->trans('site.forms.forgotPassword.errors.invalidEmail');
        $this->addFlash('error', $message);
      }
    }

    return $this->render('@Website/Security/forgotPassword.html.twig', [
      'headerTitle' => 'PREVEND',
      'form' => $form->createView()
    ]);
  }

  public function resetPasswordAction(Request $request) {
    $participantId = $request->get('participantId');
    $hash = $request->get('hash');

    //Get the participant
    $entityManager = $this->get('doctrine.orm.entity_manager');
    $entityRepo = $entityManager->getRepository(Participant::class);
    $participant = $entityRepo->findOneBy([
      'id' => $participantId,
      'forgotPasswordHash' => $hash
    ]);

    if (empty($participant)) {
      $translator = $this->get('translator');
      $message = $translator->trans('site.forms.resetPassword.error');
      $this->addFlash('error', $message);
      return $this->redirectToRoute('user_forgot_password');
    }

    $form = $this->createForm(ResetPasswordForm::class, $participant);

    //Handle the form
    $form->handleRequest($request);

    if ($form->isValid() && $form->isSubmitted()) {
      $salt = base64_encode(random_bytes(32));
      $passwordEncoder = $this->get('security.encoder_factory')->getEncoder($participant);
      $newPassword = $passwordEncoder->encodePassword(
        $participant->getPlainPassword(),
        $salt
      );

      //Update the participant
      $participant
        ->setSalt($salt)
        ->setPassword($newPassword)
        ->setForgotPasswordHash(null)
      ;

      $entityRepo->updateEntities([$participant]);

      //Set a flash message and redirect them
      $translator = $this->get('translator');
      $message = $translator->trans('site.forms.resetPassword.success');
      $this->addFlash('success', $message);

      return $this->redirectToRoute('user_login');
    }

    return $this->render('@Website/Security/resetPassword.html.twig', [
      'headerTitle' => 'PREVEND',
      'form' => $form->createView()
    ]);
  }
}
