<?php

namespace Bazookas\AdminBundle\PageBuilder;

use Bazookas\AdminBundle\AdminElements\Containers\AccordionContainer;
use Bazookas\AdminBundle\AdminElements\Containers\ListTableContainer;
use Bazookas\AdminBundle\AdminElements\Containers\PanelContainer;
use Bazookas\AdminBundle\AdminElements\Containers\RowContainer;
use Bazookas\AdminBundle\AdminElements\Elements\HeaderElement;
use Bazookas\AdminBundle\AdminElements\Elements\ListFieldElement;
use Bazookas\AdminBundle\AdminElements\Elements\ListGroupElement;
use Bazookas\AdminBundle\AdminElements\Elements\TextElement;
use Bazookas\AdminBundle\Filter\ListFilter;
use Bazookas\CommonBundle\Utility\Utility;
use Doctrine\ORM\QueryBuilder;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Translation\Translator;
use Symfony\Component\Translation\TranslatorInterface;

class CopyPageBuilder extends ListPageBuilder
{

  /**
   * @var Translator
   */
  private $translator;

  /**
   * @var string[]
   */
  private $locales;

  /**
   * @var string[]
   */
  private $translationDomains;

  /**
   * @var array
   */
  private $translationData;

  public function __construct(TranslatorInterface $translator, $locales)
  {
    $this->translator = $translator;
    $this->locales = explode('|', $locales);
    $this->translationDomains = ['messages'];
  }

  /**
   * @return \string[]
   */
  public function getTranslationDomains()
  {
    return $this->translationDomains;
  }

  /**
   * @param \string[] $translationDomains
   * @return CopyPageBuilder
   */
  public function setTranslationDomains(array $translationDomains)
  {
    $this->translationDomains = $translationDomains;

    return $this;
  }

  /**
   * @inheritDoc
   */
  public function buildPage(Request $request)
  {
    $this
      ->addElement($this->generateHeaderElement())
      ->addElement($this->generateHelpPanel())
      ->addElement($this->buildListTable($request));
  }

  /**
   * @inheritDoc
   */
  protected function generateHeaderElement()
  {
    return new RowContainer([
      new HeaderElement([], [
        'label' => 'admin.entities.copy.listLabel',
      ]),
      $this->createPageActionContainer(),
    ], [
      'defaultColClass' => 'col-md-6',
    ]);
  }

  public function generateHelpPanel() {
    return new AccordionContainer([
        new ListGroupElement([], [
          'navName' => 'admin.entities.copy.help.navName',
          'content' => [
            'admin.entities.copy.help.content.item1.header' => 'admin.entities.copy.help.content.item1.text'
          ]
        ])
    ], [
      'class' => 'panel-group accordion mar-btm',
      'initiallyCollapsed' => true
    ]);
  }

  /**
   * @inheritDoc
   */
  public function buildListTable(Request $request)
  {
    //Generate the list fields
    $this->generateListFields();

    //Add FilterFields
    $this->addListFilterFields($this->fields);

    $listTable = new ListTableContainer([], [
      'ajaxPageable' => $this->isAjaxPageable(),
      'pageAction' => $this->getPageAction(),
      'inlineEditable' => $this->isInlineEditable(),
    ]);
    $listTable->setFields($this->fields);
    $listTable->setActions($this->actions);

    //Get the current page
    $page = $request->get('page', 1);
    $page--;

    //Fill the list table container with data
    $this->getTranslationData();

    //apply filtering as needed
    $this->handleFiltering($request);

    //Convert the data to list table format
    list($pageCount, $tableData) = $this->convertTranslationDataToListTableContainerFormat($this->translationData, $page);

    //Populate the table
    $listTable
      ->setTableData($tableData)
      ->setPageCount($pageCount)
      ->setCurrentPage(++$page);

    return $listTable;
  }

  /**
   * @return ListFieldElement[]
   */
  protected function generateListFields()
  {
    $listFields = [];

    if (count($this->translationDomains) > 1) {
      $this->addField('domain', [
        'label' => 'admin.entities.copy.fields.domain',
        'class' => 'text-left',
        'inlineEditable' => false,
      ]);
    }

    $this->addField('key', [
      'label' => 'admin.entities.copy.fields.key',
      'class' => 'text-left',
      'inlineEditable' => false,
    ]);


    foreach ($this->locales as $locale) {
      $this->addField($locale, [
        'label' =>  'admin.entities.copy.enums.' . $locale,
        'class' => 'text-left',
        'inlineFieldType' => 'textarea',
      ]);
    }

    return $listFields;
  }

  /**
   * @param ListFieldElement[] $listFields
   */
  private function addListFilterFields(array $listFields)
  {
    foreach ($listFields as $field) {
      $fieldName = $field->getField();
      if ($fieldName !== 'domain') {
        $this->addTextFilterField($fieldName, [
          'label' => $field->getLabel(),
        ]);
      }
    }
  }

  /**
   * Return and create the list table container data for the given domains and languages.
   * @return array
   */
  public function getTranslationData()
  {
    //Generate an array of catalogue messages
    $messages = [];
    foreach ($this->locales as $locale) {
      $messages[$locale] = $this->translator->getCatalogue($locale);
    }

    //Generate the filtered translation data
    $translationData = [];

    //Loop through all of the defined languages
    foreach ($this->locales as $locale) {

      //Loop through all of the defined domains
      foreach ($this->translationDomains as $domain) {
        $messageData = $messages[$locale]->all();
        if (isset($messageData[$domain])) {

          //Foreach each key => translation pair, group all languages in 1 array item
          foreach ($messageData[$domain] as $key => $translation) {
            if (!isset($translationData[$domain])) {
              $translationData[$domain] = [];
            }

            //TODO to be removed in admin v2, isKeyExcluded check is only for backwards compatibility
            if (!$this->isKeyExcluded($key)) {
              if (!isset($translationData[$domain][$key])) {
                $translationData[$domain][$key] = [
                  'domain' => $domain,
                  'key' => $key,
                  'id' => $domain . ':' . $key,
                ];

                //Ensure that a key exists for each language
                foreach ($this->locales as $tempLocale) {
                  $translationData[$domain][$key][$tempLocale] = null;
                }
              }

              $translationData[$domain][$key][$locale] = $translation;
            }
          }
        }
      }
    }

    $this->translationData = $translationData;

    return $translationData;
  }

  /**
   * @deprecated
   * A admin v1 compatibility function, to be removed in v2
   * @param $key
   * @return bool
   */
  private function isKeyExcluded($key)
  {
    $excludePrefixes = ['common.', 'admin.', 'profile.', 'dashboard.', 'menu.', 'adminList.', 'youtube.', 'adminlist.', 'user.', 'registration.', 'media.'];

    foreach ($excludePrefixes as $prefix) {
      if (Utility::startsWith($key, $prefix)) {
        return true;
      }
    }

    return false;
  }

  /**
   * Overwriting parent method
   *
   * @param Request $request
   * @param QueryBuilder $qb
   */
  protected function handleFiltering(Request $request, QueryBuilder $qb = null)
  {
    $filterService = $this->getFilterService();
    $filterService
      ->setMaintainFilterState($this->getMaintainFilterState())
      ->parseFilters($request);

    //Use a custom function to carry out the actual filtering
    $this->executeFilters($filterService->getFilters());

    //create and add the filter form
    $this->addFilterForm($filterService);
  }

  /**
   * @param ListFilter[] $filters
   */
  private function executeFilters(array $filters)
  {
    foreach ($filters as $filter) {
      switch ($filter->getField()) {
        case "domain":
          $this->filterByDomain($filter);
          break;
        case "key":
          $this->filterByKey($filter);
          break;
        default:
          $this->filterByLocale($filter);
          break;
      }
    }
  }

  /**
   * @param ListFilter $filter
   */
  private function filterByDomain($filter)
  {
    $operator = $filter->getOperator();
    $condition = null;
    foreach ($this->translationData as $domain => $data) {
      $condition = $this->evaluateFilterCondition($operator, $domain, $filter->getValue());

      if ($condition) {
        unset($this->translationData[$domain]);
      }
    }
  }

  /**
   * @param string $operator
   * @param string $itemKey
   * @param string $value
   * @return bool
   */
  private function evaluateFilterCondition($operator, $itemKey, $value ) {
    $condition = false;
    switch ($operator) {
      case ListFilter::OPERATOR_LIKE:
        $condition = (strpos($itemKey, $value) === false);
        break;
      case ListFilter::OPERATOR_NOT_LIKE:
        $condition = (strpos($itemKey, $value) >= 0);
        break;
      case ListFilter::OPERATOR_EQUALS:
        $condition = ($itemKey != $value);
        break;
      case ListFilter::OPERATOR_NOT_EQUALS:
        $condition = ($itemKey === $value);
        break;
    }

    return $condition;
  }

  /**
   * @param ListFilter $filter
   */
  private function filterByKey($filter)
  {
    $operator = $filter->getOperator();
    $condition = null;
    foreach ($this->translationData as $domain => $data) {
      foreach ($data as $transKey => $trans) {
        $condition = $this->evaluateFilterCondition($operator, $transKey, $filter->getValue());

        if ($condition) {
          unset($this->translationData[$domain][$transKey]);
        }
      }
    }
  }

  /**
   * @param ListFilter $filter
   */
  private function filterByLocale($filter)
  {
    $operator = $filter->getOperator();
    $locale = $filter->getField();
    $condition = null;
    foreach ($this->translationData as $domain => $data) {
      foreach ($data as $transKey => $trans) {
        $condition = $this->evaluateFilterCondition($operator, $trans[$locale], $filter->getValue());

        if ($condition) {
          unset($this->translationData[$domain][$transKey]);
        }
      }
    }
  }

  /**
   * @param array $translationData
   * @return array
   */
  private function convertTranslationDataToListTableContainerFormat(array $translationData, $page)
  {
    //Convert the data for use with the list table container
    $data = [];
    foreach ($translationData as $domain => $domainData) {
      foreach ($domainData as $key => $translationObject) {
        $data[] = $translationObject;
      }
    }

    //calculate the offset
    $offset = $page * $this->limit;
    $pageCount = ceil(count($data) / $this->limit);

    return [$pageCount, array_slice($data, $offset, $this->limit)];
  }

  /**
   * Overwrite this as we're not using an entity
   *
   * @inheritDoc
   */
  public function getEntityShortClassName()
  {
    return 'copy';
  }

}
