<?php

namespace Bazookas\AdminBundle\Util;

use Bazookas\AdminBundle\Controller\ListBaseController;
use Bazookas\AdminBundle\Filter\Filter;
use Bazookas\AdminBundle\Filter\FilterField;

/**
 * Helper class to configure admin lists
 **/
class ListConfigurator {

  /**************************
   *    GENERAL SETTINGS    *
   **************************/
  /**
   * @var string full entity class name
   **/
  public $entity;

  public $showSearch = false;

  public $showFilter = false;

  // message translation keys
  public $addedMessage = 'admin.list.messages.added';
  public $editedMessage = 'admin.list.messages.edited';
  public $entityName = '';
  private $titles = array();

  /**
   * add custom styles
   * @var array
   */
  private $styles = [];

  /**
   * add custom scripts
   * @var array
   */
  private $scripts = [];

  /**
   * Get page titles
   * @param string the action name to get the title for
   * @return string translation key, if not found returns "admin.list.title.$action"
   **/
  public function getTitle($action) {
    if (isset($this->titles[$action])) {
      return $this->titles[$action];
    }
    return "admin.list.title.$action";
  }
  /**
   * Set a page title
   * @param string the action to set the title for
   * @param string translation key for the title
   * @return self
   **/
  public function setTitle($action, $title) {
    $this->titles[$action] = $title;
    return $this;
  }

  public function getEntity()
  {
    return $this->entity;
  }

  public function setEntity($entity)
  {
    $this->entity = $entity;

    return $this;
  }

  public function isShowSearch()
  {
    return $this->showSearch;
  }

  public function setShowSearch($showSearch)
  {
    $this->showSearch = $showSearch;

    return $this;
  }

  public function isShowFilter()
  {
    return $this->showFilter;
  }

  public function setShowFilter($showFilter)
  {
    $this->showFilter = $showFilter;

    return $this;
  }

  public function getAddedMessage()
  {
    return $this->addedMessage;
  }

  public function setAddedMessage($addedMessage)
  {
    $this->addedMessage = $addedMessage;

    return $this;
  }

  public function getEditedMessage()
  {
    return $this->editedMessage;
  }

  public function setEditedMessage($editedMessage)
  {
    $this->editedMessage = $editedMessage;

    return $this;
  }

  public function getEntityName()
  {
    if (empty($this->entityName)) {
      return 'admin.entities.' . lcfirst($this->getEntityShortClassName()). 'name';
    } else {
      return $this->entityName;
    }
  }

  public function setEntityName($entityName)
  {
    $this->entityName = $entityName;

    return $this;
  }

  public function getTitles()
  {
    return $this->titles;
  }

  public function setTitles($titles)
  {
    $this->titles = $titles;

    return $this;
  }

  public function getStyles()
  {
    return $this->styles;
  }

  public function setStyles(array $styles)
  {
    $this->styles = $styles;

    return $this;
  }

  public function getScripts()
  {
    return $this->scripts;
  }

  public function setScripts(array $scripts)
  {
    $this->scripts = $scripts;

    return $this;
  }

  /***********************
   *    ACCESS CHECKS    *
   ***********************/

  /**
   * @var array an array of access rights
   **/
  private $access = array(
    ListBaseController::ACTION_VIEW => false,
    ListBaseController::ACTION_EXPORT => false
  );
  /**
   * Checks if user can access certain action
   * @param string the action to check
   * @return boolean access set in the access array (default TRUE)
   **/
  public function getAccess($action) {
    if (!isset($this->access[$action])) {
      return TRUE;
    }
    return $this->access[$action];
  }
  /**
   * Sets access to certain action, if nothing is set the get function will return true by default
   * @param string the action to set access for
   * @param boolean
   * @return self
   **/
  public function setAccess($action, $granted) {
    $this->access[$action] = $granted;
    return $this;
  }

  /***************
   *    FORMS    *
   ***************/
  private $addForm;
  private $editForm;
  /**
   * Set the value of both Edit and Add form
   * @param AbstractType $form form
   * @return self
   */
  public function setForm($form) {
    $this->addForm = $form;
    $this->editForm = $form;
    return $this;
  }
  /**
   * Set the value of Edit Form
   * @param AbstractType $form editForm
   * @return self
   */
  public function setEditForm($editForm)
  {
    $this->editForm = $editForm;
    return $this;
  }
  /**
   * Set the value of Add Form
   * @param AbstractType $form addForm
   * @return self
   */
  public function setAddForm($addForm)
  {
    $this->addForm = $addForm;
    return $this;
  }
  /**
   * @return AbstractType
   */
  public function getAddForm() {
    return $this->addForm;
  }
  /**
   * @return AbstractType
   */
  public function getEditForm() {
    return $this->editForm;
  }

  /***************
   *    VIEWS    *
   ***************/
  /**
   * @var string the template to use for the list page
   **/
  public $listTemplate;

  /**
   * @var string template to use for the edit view
   **/
  public $editTemplate;

  /**
   * @var string template to use for add view (same as edit view in most cases)
   **/
  public $addTemplate;

  /**
   * @var string template to use for view view
   **/
  public $viewTemplate;

  public $breadcrumbTemplate;

  public function getListTemplate()
  {
    return $this->listTemplate;
  }

  public function setListTemplate($listTemplate)
  {
    $this->listTemplate = $listTemplate;

    return $this;
  }

  public function getEditTemplate()
  {
    return $this->editTemplate;
  }

  public function setEditTemplate($editTemplate)
  {
    $this->editTemplate = $editTemplate;

    return $this;
  }

  public function getAddTemplate()
  {
    return $this->addTemplate;
  }

  public function setAddTemplate($addTemplate)
  {
    $this->addTemplate = $addTemplate;

    return $this;
  }

  public function getViewTemplate()
  {
    return $this->viewTemplate;
  }

  public function setViewTemplate($viewTemplate)
  {
    $this->viewTemplate = $viewTemplate;

    return $this;
  }

  public function getBreadcrumbTemplate()
  {
    return $this->breadcrumbTemplate;
  }

  public function setBreadcrumbTemplate($breadcrumbTemplate)
  {
    $this->breadcrumbTemplate = $breadcrumbTemplate;

    return $this;
  }

  /*******************
   *    LIST VIEW    *
   *******************/
  public $orderBy = 'id';
  public $orderDirection = 'DESC';
  public $currentPage = 1;
  public $totalPages;
  public $limit = 50;

  /**
   * @var array Define which fields to show in the overview table
   **/
  public $listFields = array();

  /**
   * @var array Define which filters to show in the overview table
   **/
  public $listFilters = array();

  /**
   * @var array define single entity actions, shown in actions column
   **/
  public $actions = array();
  public $pageActions = array();
  private $orderable = false;
  private $updateOrderRoute;
  private $maintainFilterState = false;

  public function getOrderBy()
  {
    return $this->orderBy;
  }

  public function setOrderBy($orderBy)
  {
    $this->orderBy = $orderBy;

    return $this;
  }

  public function getOrderDirection()
  {
    return $this->orderDirection;
  }

  public function setOrderDirection($orderDirection)
  {
    $this->orderDirection = $orderDirection;

    return $this;
  }

  public function getCurrentPage()
  {
    return $this->currentPage;
  }

  public function setCurrentPage($currentPage)
  {
    $this->currentPage = $currentPage;

    return $this;
  }

  public function getTotalPages()
  {
    return $this->totalPages;
  }

  public function setTotalPages($totalPages)
  {
    $this->totalPages = $totalPages;

    return $this;
  }

  public function getLimit()
  {
    return $this->limit;
  }

  public function setLimit($limit)
  {
    $this->limit = $limit;

    return $this;
  }

  public function getListFields()
  {
    return $this->listFields;
  }

  public function setListFields($listFields)
  {
    $this->listFields = $listFields;

    return $this;
  }

  public function getListFilters()
  {
    return $this->listFilters;
  }

  public function setListFilters($listFilters)
  {
    $this->listFilters = $listFilters;

    return $this;
  }

  public function getActions()
  {
    return $this->actions;
  }

  public function setActions($actions)
  {
    $this->actions = $actions;

    return $this;
  }

  public function getPageActions()
  {
    return $this->pageActions;
  }

  public function setPageActions($pageActions)
  {
    $this->pageActions = $pageActions;

    return $this;
  }

  public function isOrderable()
  {
    return $this->orderable;
  }

  public function setOrderable($orderable)
  {
    $this->orderable = $orderable;

    return $this;
  }

  public function getUpdateOrderRoute()
  {
    return $this->updateOrderRoute;
  }

  public function setUpdateOrderRoute($updateOrderRoute)
  {
    $this->updateOrderRoute = $updateOrderRoute;

    return $this;
  }

  public function getMaintainFilterState()
  {
    return $this->maintainFilterState;
  }

  public function setMaintainFilterState($maintainFilterState)
  {
    $this->maintainFilterState = $maintainFilterState;

    return $this;
  }

  /**
   * Add a field to the list of fields to display
   * @param string|ListField The property name of the field to render or a ListField instance, if the latter is given, other params are skipped
   * @param string label to use in table header
   * @param boolean|string whether or not the column is sortable, if string is given, this will be used as field to do sorting
   * @param string path to the template to render the field
   * @return self
   **/
  public function addField($fieldName, $label = null, $sortable = false, $class = 'text-center', $template = null) {
    if (!$sortable) {
      $this->addBasicField($fieldName, $label,  $template, $class);
    } else {
      $this->addSortableField($fieldName, $label,  $template, $class);
    }

    return $this;
  }

  protected function createField($fieldName, $label, $template, $class, $sortable, $orderable) {
    $field = new ListField();
    $field
      ->setField($fieldName)
      ->setLabel($label)
      ->setTemplate($template)
      ->setClass($class)
      ->setSortable($sortable)
      ->setOrderable($orderable)
    ;

    return $field;
  }

  public function addBasicField($fieldName, $label = null,  $template = null, $class = 'text-center') {
    if ($fieldName instanceof ListField) {
      $field = $fieldName;
    }
    else {
      if (empty($label)) {
        $label = 'admin.entities.' . lcfirst($this->getEntityShortClassName()) . '.fields.' . $fieldName;
      }

      $field = $this->createField($fieldName, $label, $template, $class, false, false);
    }

    $this->listFields[] = $field;

    return $this;
  }

  public function addBooleanField($fieldName, $label = null, $class = 'text-center' ) {
    $this->addBasicField($fieldName, $label , 'BazookasAdminBundle:List/Fields:_boolean.html.twig', $class);
    return $this;
  }

  public function addDateField($fieldName, $label = null, $class = 'text-center' ) {
    $this->addBasicField($fieldName, $label , 'BazookasAdminBundle:List/Fields:_date.html.twig', $class);
    return $this;
  }

  public function addDateTimeField($fieldName, $label = null, $class = 'text-center' ) {
    $this->addBasicField($fieldName, $label , 'BazookasAdminBundle:List/Fields:_dateTime.html.twig', $class);
    return $this;
  }

  public function addImageField($fieldName, $label = null, $class = 'text-center' ) {
    $this->addBasicField($fieldName, $label , 'BazookasAdminBundle:List/Fields:_image.html.twig', $class);
    return $this;
  }

  public function addImageIconField($fieldName, $label = null, $class = 'text-center' ) {
    $this->addBasicField($fieldName, $label , 'BazookasAdminBundle:List/Fields:_imageIcon.html.twig', $class);
    return $this;
  }

  public function addSortableField($fieldName, $label = null,  $template = null, $class = 'text-center') {
    if ($fieldName instanceof ListField) {
      $field = $fieldName;
    }
    else {
      if (empty($label)) {
        $label = 'admin.entities.' . lcfirst($this->getEntityShortClassName()) . '.fields.' . $fieldName;
      }

      $field = $this->createField($fieldName, $label, $template, $class, true, false);
    }

    $this->listFields[] = $field;

    return $this;
  }

  public function addOrderableField($fieldName, $label = null,  $template = null, $class = 'text-center') {
    $this->setOrderable(true);

    if ($fieldName instanceof ListField) {
      $field = $fieldName;
    }
    else {
      if (empty($label)) {
        $label = 'admin.entities.' . lcfirst($this->getEntityShortClassName()) . '.fields.' . $fieldName;
      }

      $field = $this->createField($fieldName, $label, $template, $class, true, true);
    }

    array_unshift($this->listFields, $field);

    return $this;
  }

  /**
   * Add a field to the list of fields to display
   * @param string|ListField The property name of the field to render or a ListField instance, if the latter is given, other params are skipped
   * @param string label to use in table header
   * @return self
   **/
  public function addFilter($fieldName, $label = null, $type = 'text', $operators = null) {
    if ($fieldName instanceof FilterField) {
      $filter = $fieldName;
    }
    else {
      $filter = $this->createFilter($fieldName, $label, $type, $operators);
    }

    $this->listFilters[] = $filter;
    return $this;
  }

  public function createFilter($fieldName, $label = null, $type = 'text', $operators = null) {
    $filter = new FilterField();
    $filter
      ->setField($fieldName)
      ->setLabel($label)
      ->setType($type)
      ->setOperators($operators)
    ;

    return $filter;
  }

  /**
   * TODO deprecate and rename to addNumberFilter
   */
  public function addBasicTextFilter($fieldName, $label = null) {
    if ($fieldName instanceof FilterField) {
      $filter = $fieldName;
    }  else {
      if (empty($label)) {
        $label = 'admin.entities.' . lcfirst($this->getEntityShortClassName()) . '.fields.' . $fieldName;
      }

      $operators = array(
        'admin.filter.operator.like' => Filter::OPERATOR_LIKE,
        'admin.filter.operator.notLike' => Filter::OPERATOR_NOT_LIKE,
        'admin.filter.operator.equals' => Filter::OPERATOR_EQUALS,
        'admin.filter.operator.notEquals' => Filter::OPERATOR_NOT_EQUALS
      );

      $filter = $this->createFilter($fieldName, $label, FilterField::TYPE_TEXT, $operators);
    }

    $this->listFilters[] = $filter;
    return $this;
  }

  /**
   * TODO deprecate and rename to addNumberFilter
   */
  public function addBasicChoiceFilter($fieldName, $label = null) {

    //TODO implement this
    throw new \Exception('Not implemented ¯\_(ツ)_/¯');

  }
  
  public function addBooleanFilter($fieldName, $label = null) {
    if ($fieldName instanceof FilterField) {
      $filter = $fieldName;
    }  else {
      if (empty($label)) {
        $label = 'admin.entities.' . lcfirst($this->getEntityShortClassName()) . '.fields.' . $fieldName;
      }

      $operators = array(
        'admin.filter.operator.equals' => Filter::OPERATOR_EQUALS
      );

      $filter = $this->createFilter($fieldName, $label, FilterField::TYPE_BOOLEAN, $operators);
    }

    $this->listFilters[] = $filter;
    return $this;
  }
  
  /**
   * @deprecated
   */
  public function addBasicBooleanFilter($fieldName, $label = null) {
    return $this->addBooleanFilter($fieldName, $label);
  }

  /**
   * TODO deprecate and rename to addNumberFilter
   */
  public function addBasicNumberFilter($fieldName, $label = null) {
    if ($fieldName instanceof FilterField) {
      $filter = $fieldName;
    }  else {
      if (empty($label)) {
        $label = 'admin.entities.' . lcfirst($this->getEntityShortClassName()) . '.fields.' . $fieldName;
      }

      $operators = array(
        'admin.filter.operator.equals' => Filter::OPERATOR_EQUALS,
        'admin.filter.operator.notEquals' => Filter::OPERATOR_NOT_EQUALS,
        'admin.filter.operator.greaterThan' => Filter::OPERATOR_GREATER_THAN,
        'admin.filter.operator.greaterThanEquals' => Filter::OPERATOR_GREATER_THAN_EQUALS,
        'admin.filter.operator.lessThan' => Filter::OPERATOR_LESS_THAN,
        'admin.filter.operator.lessThanEquals' => Filter::OPERATOR_LESS_THAN_EQUALS,
      );

      $filter = $this->createFilter($fieldName, $label, FilterField::TYPE_NUMBER, $operators);
    }

    $this->listFilters[] = $filter;
    return $this;
  }

  public function getEntityShortClassName() {
    if (strpos($this->getEntity(), '\\')) {
      $split = explode('\\', $this->getEntity());
    } else {
      $split = explode(':', $this->getEntity());
    }

    return $split[count($split) - 1];
  }

  /**
   * Checks if a field by this name exists in the configuration
   **/
  public function getFieldByName($fieldName) {
    foreach ($this->listFields as $field) {
      if ($field->field === $fieldName) {
        return $field;
      }
    }
    return null;
  }
  /**
   * Checks if is sortable by it's name
   **/
  public function getFieldSortable() {
    $field = $this->getFieldByName();
    if (empty($field)) {
      return false;
    }
    return $field->sortable;
  }

  /**
   * Add an action
   * @param string template the location of the button template to be rendered
   * @param string route the
   * @return self
   **/
  public function addAction($template, $route) {
    $action = new ListAction($route, $template);

    $this->actions[] = $action;
    return $this;
  }

}
