# Using multiple facebook-apps

There is a possibility that you want to use the facebook authentication, but your project contains multiple facebook-apps. For that reason it is impossible to place your facebookAppId and facebookAppSecret in the config.yml.

This page shows a possible solution for this problem.

### 1. Create a factory class for the facebook-service

Your factory class should look something like this.

```php
class FacebookServiceFactory {
  public static function createFacebookService(RequestStack $requestStack, OAuthStorage $tokenStorage, array $settings) {
    $clientId = $requestStack->getCurrentRequest()->get('client_id');
    /** @var OAuth2Client $client */
    $client = $tokenStorage->getClient($clientId);
    /** @var App $app */
    $app = $client->getApp();

    // check if the app is still active
    if ($app->getActive()) {
      $settings['appId'] = $app->getFacebookAppId();
      $settings['appSecret'] = $app->getFacebookAppSecret();

      $service = new FacebookService($settings);
      return $service;
    } else {
      throw new ApiException(
        'The app has been disabled and no longer has access to the API. Please contact support to resolve this issue.',
        ApiExceptionMap::UNAUTHORIZED,
        Response::HTTP_FORBIDDEN
      );
    }
  }
}
```

### 2. Update services.yml

You will need to add both the facebook-grant-extension and the facebook-service in your services.yml. The facebook-service contains the factory and factory-method that are called when a facebook-service is created.

```php
    APIBundle.Security.FacebookService:
      class:     Bazookas\IntegrationsBundle\Services\Facebook\FacebookService
      factory: ['APIBundle\Security\FacebookServiceFactory', createFacebookService]
      arguments:
        - "@request_stack"
        - '@fos_oauth_server.storage'
        - { facebookIdField: '%bazookas_integrations.facebook.facebookIdField%', fields: '%bazookas_integrations.facebook.fields%' }
      lazy: true

    apiframeworkbundle.oauth.facebook_grant_extension:
      class: Bazookas\APIFrameworkBundle\Security\OAuth\GrantTypeExtensions\FacebookOAuthGrantExtension
      arguments:
        - "@apibundle.user_provider.baseuserprovider"
        - "@APIBundle.Security.FacebookService"
        - false
      tags:
        - { name: fos_oauth_server.grant_extension, uri: "http://facebook.com" }
      lazy: true

```
